# Reverse engineering of existing solutions

## Setting up the project

As this implementation will be a very rough representation of what's actually
expected of me by the end of this thing, I'll write everything (up to a certain
point) inside a single `.c` source file (`hypervisor.c`). The main reason for
this is to simplify the management of different [translation units](https://en.wikipedia.org/wiki/Translation_unit_(programming)).
inside of my `Makefile`.

Speaking of `Makefile`s, mine will be based off the one presented in this[^1]
great guide about the development of LKMs. As I'm writing this (12/02/2024),
this is how it's looking so far: 

```make
obj-m += hypervisor.o 

PWD := $(CURDIR) 

all:
    $(MAKE) -C /lib/modules/$(shell uname -r)/build M=$(PWD) modules 

clean:
    $(MAKE) -C /lib/modules/$(shell uname -r)/build M=$(PWD) clean
```

I tried using the `Makefile` given in the tutorial[^2], but at a first glance
it seemed more "complicated" and most importantly, it didn't _automagically_
cleanup all the compilation products (e.g. `.o`, `.ko`, `.symvers`, etc...).

The ins and outs of the `Makefile` shown above, will be studied thoroughly at
a later stage.

## Inline assembly in `C`

> _The `asm` keyword may be used in place of `__asm__`, however `__asm__` is
> portable whereas the `asm` keyword is a `GNU` extension. In further examples I
> will only use the `__asm__` variant._[^3]

### Basic syntax

```c
asm ( assembler template
    : output operands                   (optional)
    : input operands                    (optional)
    : clobbered registers list          (optional)
    );
```

#### Example

```c
int a=10, b;
asm ("movl %1, %%eax; 
      movl %%eax, %0;"
     :"=r"(b)        /* output */
     :"r"(a)         /* input */
     :"%eax"         /* clobbered register */
     );
```

### Useful references

- [OS Dev -- Inline assembly](https://wiki.osdev.org/Inline_Assembly)
- [0xax](https://0xax.gitbooks.io/linux-insides/content/Theory/linux-theory-3.html)

## Entering VMX

### Discovering support for VMX

> _"System software can determine whether a processor supports VMX operation using CPUID. If
> CPUID.1:ECX.VMX[bit 5] = 1, then VMX operation is supported"._[^4]

In Intel's nomenclature, I presume the statement `CPUID.1:ECX.VMX[bit 5]` can
be decoded the following way:

- `CPUID.`: Instruction to execute
- `1:`: Argument given to the `a` register (e.g. `rax` on a 64-bit architecture)
to the above mentioned instruction
- `ECX`: Register containing the return value
- `[bit 5]`: The bit whose value we're concerned about

```c
static bool vmx_supported(void) {
    /*unsigned int ecx = cpuid_ecx(1);*/ // #include "asm/cpuid.h"
    int ecx;

    __asm__ volatile("mov $1, %rax");
    __asm__ volatile("cpuid");
    __asm__ volatile("mov %%ecx , %0\n\t" : "=r"(ecx));

    return (ecx >> 5) & 1;
}
```

### Enabling VMX in CR4

> [!IMPORTANT]
> Looks like, when virtualization has been activated through the BIOS, the 13th
> bit of CR4 is **always** set. Thus, setting this bit manually seems to be a
> no-op.

In order to enter VMX operation, the software (our hypervisor) has to firstly
enable VMX by setting the 13th bit of the CR4 register to 1. This will of course
throw an exception if this code is being executed from user-space. That's why
this function can only be tested when being ran at the kernel level (e.g. ring 0).

```c
static void cr4_enable_vmx(void) {
    unsigned long cr4;

    __asm__ volatile("mov %%cr4, %0" : "=r"(cr4)::"memory");
    cr4 |= (1 << 13);
    __asm__ volatile("mov %0, %%cr4" ::"r"(cr4) : "memory");
}
```

### IA32_FEATURE_CONTROL MSR

VMXON is also controlled by the IA32_FEATURE_CONTROL MSR (Model-Specific
Register). MSRs are 64-bit wide. The address of IA32_FEATURE_CONTROL MSR is
**0x3a**.

The relevant bits (in terms of VMX) of this MSR are:

- Bit 0 (Lock bit)
    - System BIOS can use this bit to provide a setup option for BIOS to disable
    support for VMX. **If this bit is set, WRMSR to this MSR will cause a general-
    protection exception**. To enable VMX support in a platform, BIOS must set bit
    1, bit 2, or both (see below), as well as the lock bit.
- Bit 1 (VMXON in SMX operation)
- Bit 2 (VMXON outside SMX operation)

**In my case, after having activated VT-x through the BIOS, the value returned
by the IA32_FEATURE_CONTROL MSR is always equal to 5 (e.g. the lock bit and
VMXON outside SMX operation are always set).**

**If I decide to deactivate VT-x, the MSR will return the value of 1 (e.g. lock
bit).**

Due to the fact that the **lock bit** is always **set**, it means that I'm not
able to write to this MSR using __wrmsr. What's interesting about this is that
the tutorial tries to modify this MSR but if we assume that the **lock bit is set**
that branch **will be a no-op**.

```c
static bool ia32_feature_control_flags(void) {
    int msr_value = __rdmsr(IA32_FEATURE_CONTROL_MSR);

    if (!(msr_value & IA32_FEATURE_CONTROL_LOCK_BIT)) {
        printk(KERN_INFO "Writing to the IA32_FEATURE_CONTROL MSR\n");
        __wrmsr(IA32_FEATURE_CONTROL_MSR,
                IA32_FEATURE_CONTROL_LOCK_BIT |
                    IA32_FEATURE_CONTROL_VMXON_OUTSIDE_SMX,
                0);
    }

    if (!(msr_value & IA32_FEATURE_CONTROL_VMXON_OUTSIDE_SMX)) {
        printk(KERN_INFO "Virtualization isn't available\n");
        return false;
    }

    return true;
}
```

### Restrictions on VMX operation

From what I understand, certains bits in the CR0 and CR4 control registers need
to either be set or cleared (still quite vague for now) so that a call to
`vmxon` won't fail.

"Full" explanation from Intel's SDM (Chapter 25.8, p. 4040):

> _In VMX operation, processors may fix certain bits in CR0 and CR4 to specific values and not support other
> values. VMXON fails if any of these bits contains an unsupported value (see “VMXON—Enter VMX Operation” in
> Chapter 32). Any attempt to set one of these bits to an unsupported value while in VMX operation (including
> VMX root operation) using any of the CLTS, LMSW, or MOV CR instructions causes a general-protection
> exception. VM entry or VM exit cannot set any of these bits to an unsupported value. Software should consult
> the VMX capability MSRs IA32_VMX_CR0_FIXED0 and IA32_VMX_CR0_FIXED1 to determine how bits in CR0
> are fixed (see Appendix A.7). For CR4, software should consult the VMX capability MSRs
> IA32_VMX_CR4_FIXED0 and IA32_VMX_CR4_FIXED1 (see Appendix A.8)._


Useful reference[^6]

```c
// Function to check if VMXON can be executed without a protection exception
static void restrictions_cr_msrs(void) {
    unsigned long long cr0, cr4;

    __asm__ volatile("mov %%cr0, %0" : "=r"(cr0));
    __asm__ volatile("mov %%cr4, %0" : "=r"(cr4));

    unsigned long long cr0_fixed0 = __rdmsr(IA32_VMX_CR0_FIXED0);
    unsigned long long cr0_fixed1 = __rdmsr(IA32_VMX_CR0_FIXED1);

    unsigned long long cr4_fixed0 = __rdmsr(IA32_VMX_CR4_FIXED0);
    unsigned long long cr4_fixed1 = __rdmsr(IA32_VMX_CR4_FIXED1);

    pr_debug("CR0 = 0x%llx\n", cr0);
    pr_debug("CR4 = 0x%llx\n", cr4);

    // If bit X is 1 in IA32_VMX_CR0_FIXED0, then that bit of CR0 is fixed to 1
    // in VMX operation. Similarly, if bit X is 0 in IA32_VMX_CR0_FIXED1, then
    // that bit of CR0 is fixed to 0 in VMX operation. It is always the case

    // Setting mandatory bits of FIXED0 in CR0
    cr0 |= cr0_fixed0;
    // Clearing mandatory bits of FIXED1 in CR0;
    cr0 &= cr0_fixed1;

    cr4 |= cr4_fixed0;
    cr4 &= cr4_fixed1;

    __asm__ volatile("mov %0, %%cr0" ::"r"(cr0));
    __asm__ volatile("mov %0, %%cr4" ::"r"(cr4));
}
```

#### CR0

![Meaning of the bits of the CR0 register](./figs/re/cr0.png)[^5]

#### CR4

![Meaning of the bits of the CR4 register](./figs/re/cr4.png)[^5]

#### Pseudo-code for successfully executing VMXON

p.4233

```
IF (register operand) or (CR0.PE = 0) or (CR4.VMXE = 0) or (RFLAGS.VM = 1) or (IA32_EFER.LMA = 1 and CS.L = 0)
THEN #UD;
ELSIF not in VMX operation
    THEN
        IF (CPL > 0) or (in A20M mode) or
        (the values of CR0 and CR4 are not supported in VMX operation; see Section 25.8) or
        (bit 0 (lock bit) of IA32_FEATURE_CONTROL MSR is clear) or
        (in SMX operation2 and bit 1 of IA32_FEATURE_CONTROL MSR is clear) or
        (outside SMX operation and bit 2 of IA32_FEATURE_CONTROL MSR is clear)
            THEN #GP(0);
            ELSE
                addr := contents of 64-bit in-memory source operand;
                IF addr is not 4KB-aligned or
                addr sets any bits beyond the physical-address width3
                    THEN VMfailInvalid;
                    ELSE
                        rev := 32 bits located at physical address addr;
                        IF rev[30:0] ≠ VMCS revision identifier supported by processor OR rev[31] = 1
                            THEN VMfailInvalid;
                            ELSE
                                current-VMCS pointer := FFFFFFFF_FFFFFFFFH;
                                enter VMX operation;
                                block INIT signals;
                                block and disable A20M;
                                clear address-range monitoring;
                                IF the processor supports Intel PT but does not allow it to be used in VMX operation 1
                                    THEN IA32_RTIT_CTL.TraceEn := 0;
                                FI;
                                VMsucceed
                        FI;
                FI; 
        FI;
ELSIF in VMX non-root operation
    THEN VMexit;
ELSIF CPL > 0
    THEN #GP(0);
    ELSE VMfail(“VMXON executed in VMX root operation”);
FI;
```

### VMXON region

Ref: §26.11.5 -- pp. 4074-4075 

Prior to entering VMX "mode of operation", it is necessary to allocate a region
in memory called the VMXON region whose physical address will be the operand
given to the `vmxon` instruction. This region (allocation that is) has to adhere
to a number of requirements:

- 4-KB aligned (aka page-aligned)
- Its size shall be specified by reading the `IA32_VMX_BASIC[44:32]` MSR (p. 4586)

<!--TODO: how to alloc a page-aligned region ??-->
> Need to investigate the different allocation strategies[^7] offered by the
> kernel.

```c
#define REGION_SIZE ((__rdmsr(IA32_VMX_BASIC) >> 32) & 0x1fff)

struct vmxon_region_t {
    void *va;
    unsigned long long pa;
} __attribute__((packed));

static int vmxon_region_alloc(void) {
    void *region = kzalloc(REGION_SIZE, GFP_KERNEL);

    if (!region) {
        return -1;
    }

    vmxon_region.va = region;
    vmxon_region.pa = __pa(region);

    return 0;
}
```

### `vmxon` execution

#### Logs

```bash
[ 1851.028954] VMX is supported!
[ 1851.028956] VMX has been successfully enabled!
[ 1851.028957] Check the necessary flags of the IA32_FEATURE_CONTROL_MSR
[ 1851.028957] IA32_FEATURE_CONTROL MSR flags allow virtualization
[ 1851.028958] Patching CR0 and CR4 depending on the value of their respective MSRs
[ 1851.028958] CR0 = 0x80050033
[ 1851.028959] CR4 = 0xf72ef0
[ 1851.028960] Allocating memory for VMXON region
[ 1851.028960] VA of the allocated region = 0xffff9dcd0d342800
[ 1851.028961] PA of the allocated region = 0x18d342800
[ 1851.028962] Reading IA32_VMX_BASIC MSR for VMCS revision identifier
[ 1851.028962] IA32_VMX_BASIC_MSR = 0x1da050000000013
[ 1851.028963] Region size to allocate = 1280 bytes
[ 1851.028963] VMCS revision identifier = 0x13
[ 1851.028964] Writing VMCS Revision ID to VMXON
[ 1851.028964] BEFORE: Reading VMXON region for VMCS ID: 0x0
[ 1851.028965] AFTER: Reading VMXON region for VMCS ID: 0x13
[ 1851.028965] Executing VMXON with address = 0x18d342800 as its operand
```

```bash
================================= FAILING =================================

[ 1312.331549] [hypervisor.c:85 in my_init()]: Querying CS (lowest 2 bits) for CPL
[ 1312.331552] [hypervisor.c:91 in my_init()]: CPL level = 0
[ 1312.331553] Checking VMX support using CPUID
[ 1312.331554] [hypervisor.c:46 in vmx_support_cpuid()]: Vendor ID = GenuineIntel
[ 1312.331555] [hypervisor.c:56 in vmx_support_cpuid()]: Physical address width = 39
[ 1312.331555] [hypervisor.c:102 in my_init()]: IA32_VMX_BASIC_MSR = 0x1da050000000013
[ 1312.331556] Initializing VMXON region
[ 1312.331557] [vmxon_reg.c:26 in alloc_vmxon_internals()]: Reading IA32_VMX_BASIC MSR for allocation size (in bytes) = 1280 
[ 1312.331558] [vmxon_reg.c:43 in write_vmcs_rev_id_to_vmxon()]: Reading IA32_VMX_BASIC MSR for VMCS revision identifier = 0x13
[ 1312.331559] [vmxon_reg.c:47 in write_vmcs_rev_id_to_vmxon()]: Writing VMCS Revision ID to VMXON region
[ 1312.331559] VA of the allocated region = 0xffff8e7329e24000
[ 1312.331560] PA of the allocated region = 0x1e9e24000
[ 1312.331561] Reading VMXON region for VMCS ID: 0x13
[ 1312.331561] Patching CR0 and CR4 depending on the value of their respective MSRs
[ 1312.331562] [msr.c:39 in patch_control_registers()]: Pre-patch: CR0 = 0x80050033
[ 1312.331562] [msr.c:40 in patch_control_registers()]: Pre-patch: CR4 = 0xf72ef0
[ 1312.331563] [msr.c:54 in patch_control_registers()]: Post-patch: CR0 = 0x80050033
[ 1312.331564] [msr.c:55 in patch_control_registers()]: Post-patch: CR4 = 0xf72ef0
[ 1312.331565] Enabling VMX in CR4
[ 1312.331565] [hypervisor.c:36 in cr4_enable_vmx()]: CR4[13].VMXE set? true
[ 1312.331566] Checking the necessary flags of the IA32_FEATURE_CONTROL_MSR
[ 1312.331566] [msr.c:8 in ia32_feature_control_flags()]: IA32_FEATURE_CONTROL value = 5
[ 1312.331567] Executing VMXON with address = 0x1e9e24000 as its operand
[ 1312.331567] [hypervisor.c:76 in vmxon()]: RFLAGS: 0x203
[ 1312.331568] `vmxon` failed with return code 1

================================= SUCCEEDING =================================

[ 1319.298789] [hypervisor.c:85 in my_init()]: Querying CS (lowest 2 bits) for CPL
[ 1319.298792] [hypervisor.c:91 in my_init()]: CPL level = 0
[ 1319.298793] Checking VMX support using CPUID
[ 1319.298793] [hypervisor.c:46 in vmx_support_cpuid()]: Vendor ID = GenuineIntel
[ 1319.298794] [hypervisor.c:56 in vmx_support_cpuid()]: Physical address width = 39
[ 1319.298795] [hypervisor.c:102 in my_init()]: IA32_VMX_BASIC_MSR = 0x1da050000000013
[ 1319.298796] Initializing VMXON region
[ 1319.298797] [vmxon_reg.c:26 in alloc_vmxon_internals()]: Reading IA32_VMX_BASIC MSR for allocation size (in bytes) = 1280 
[ 1319.298798] [vmxon_reg.c:43 in write_vmcs_rev_id_to_vmxon()]: Reading IA32_VMX_BASIC MSR for VMCS revision identifier = 0x13
[ 1319.298798] [vmxon_reg.c:47 in write_vmcs_rev_id_to_vmxon()]: Writing VMCS Revision ID to VMXON region
[ 1319.298799] VA of the allocated region = 0xffff8e733b1d4000
[ 1319.298800] PA of the allocated region = 0x1fb1d4000
[ 1319.298800] Reading VMXON region for VMCS ID: 0x13
[ 1319.298801] Patching CR0 and CR4 depending on the value of their respective MSRs
[ 1319.298801] [msr.c:39 in patch_control_registers()]: Pre-patch: CR0 = 0x80050033
[ 1319.298802] [msr.c:40 in patch_control_registers()]: Pre-patch: CR4 = 0xf72ef0
[ 1319.298803] [msr.c:54 in patch_control_registers()]: Post-patch: CR0 = 0x80050033
[ 1319.298803] [msr.c:55 in patch_control_registers()]: Post-patch: CR4 = 0xf72ef0
[ 1319.298804] Enabling VMX in CR4
[ 1319.298804] [hypervisor.c:36 in cr4_enable_vmx()]: CR4[13].VMXE set? true
[ 1319.298805] Checking the necessary flags of the IA32_FEATURE_CONTROL_MSR
[ 1319.298805] [msr.c:8 in ia32_feature_control_flags()]: IA32_FEATURE_CONTROL value = 5
[ 1319.298806] Executing VMXON with address = 0x1fb1d4000 as its operand
[ 1319.298807] [hypervisor.c:76 in vmxon()]: RFLAGS: 0x202
[ 1319.298808] `vmxon` was successfully executed!
```

## References

[^1]: [The Linux Kernel Module Programming Guide](https://sysprog21.github.io/lkmpg/)
[^2]: [Developing a hypervisor from scratch](https://nixhacker.com/developing-hypervisior-from-scratch-part-1/)
[^3]: [0xax](https://0xax.gitbooks.io/linux-insides/content/Theory/linux-theory-3.html)
[^4]: [Intel SDM 2024 Vol. 3](https://cdrdv2.intel.com/v1/dl/getContent/671447)
[^5]: [CPU Registers x86](https://wiki.osdev.org/CPU_Registers_x86)
[⁶]: [The basics of Intel VT-x extensions](https://research.meekolab.com/the-basics-of-intel-vt-x-extensions#heading-vmxon-execution)
[^7]: [Memory Allocation Guide](https://www.kernel.org/doc/html/v6.12/core-api/memory-allocation.html#memory-allocation-guide)
